#include <iostream>
#include <fstream>
#include <cstring>
#include <cstdlib>
#include <sstream>
#include <vector>



#ifndef __random_h__
#define __random_h__

#include <cmath>
#include <cstdio>
#include <cstdlib>

const int MBIG = 1000000000, MSEED = 161803398;
const double FAC = 1.0 / MBIG;

int inext, inextp;
int ra [56];

int nextrand (void)
{
  int j;
  if (++inext == 56) inext = 1;
  if (++inextp == 56) inextp = 1;
  j = ra[inext] - ra[inextp];
  if (j < 0) j += MBIG;
  ra[inext] = j;
  return j;
}

bool initrand (int seed)
{
  int i, j, k, l;
  l = MSEED - seed;
  if (l < 0) l = -l;
  l %= MBIG;
  ra[55] = l;
  k = 1;
  for (i = 1, j = 0; i <= 54; i++)
  {
   j += 21;
   if (j > 55) j -= 55;
   ra[j] = k;
   k = l - k;
   if (k < 0) k += MBIG;
   l = ra[j];
  }
  for (k = 1; k <= 4; k++)
   for (i = 1, j = 31; i <= 55; i++, j >= 55 ? j = 1 : j++)
   {
    ra[i] -= ra[j];
    if (ra[i] < 0) ra[i] += MBIG;
   }
  inext = 0;
  inextp = 31;
  return true;
}

double rndvalue (void)
{
  return nextrand () * FAC;
}

int rndvalue (int k)
{
  return (int) floor (rndvalue () * k);
}

long long Time()
{
#ifdef __GNUC__
  long long res;
  asm volatile ("rdtsc" : "=A" (res));
  return res;
#else
  int lo, h;
  __asm{
    rdtsc
    mov lo, eax
    mov h, edx
  }
  return (((long long)h) << 32ll) | lo;
#endif
}

#define R(A, B) ((A) + rndvalue((B) - (A) + 1))

#endif // __random_h__


using namespace std;

int NUMPOSWGT = 50;
int NUMNEGWGT = 1;

int RGood(int hi)
{
	int a = rndvalue(1000000000) << 16 | rndvalue(1000000000);
	if (a < 0) a = -a;
	if (a < 0) a = 0;
	a %= hi;
	return a;
}

int RGood(int lo, int hi)
{	
	return lo + RGood(hi - lo + 1);
}

int curtest;

void OpenNextFile()
{
	curtest++;
	char fname[10];
    sprintf(fname, "%.3d", curtest);
    cerr << "Writing test " << fname << "\n";
    freopen(fname, "wt", stdout);
}

void CloseFile()
{
	fflush(stdout);
	fclose(stdout);
}

const int MAXN = 100000;
const int MAXM = 500000;
const int MAXC = 1000000000;

class Edge
{
public:
	int be, en;
	int c;
};

void GenSampleTests()
{
	OpenNextFile();
	cout << "2\n1\n1 2\n1\n2 1 40\n";
	CloseFile();
	OpenNextFile();
	cout << "2\n1\n1 2\n0\n";
	CloseFile();
	OpenNextFile();
	cout << "4\n5\n1 2\n1 3\n2 3\n2 4\n3 4\n2\n4 2 1\n3 1 1\n";
	CloseFile();
}

int first[2 * MAXN + 1];
int next[2 * MAXM];
Edge edg[2 * MAXM];
int ne;
int nver;
int layer[2 * MAXN + 1];
int layerbe[2 * MAXN + 1];
int layeren[2 * MAXN + 1];
int numlayers;
int havein[2 * MAXN + 1];
int ne2;
Edge edg2[MAXM];

void Init()
{
	ne = nver = ne2 = 0;
}

void AddEdge(int v1, int v2)
{
	if (ne > MAXM) return;
	edg[ne].be = v1;
	edg[ne].en = v2;
	ne++;
}

void GenLayered(int nlay, int inlayl, int inlayu, int minae, int maxae)
{
	nver = 2;
	layer[1] = 1;
	layerbe[1] = layeren[1] = 1;
	layer[2] = nlay;
	layerbe[nlay] = layeren[nlay] = 2;
	int i, j;
	int plb, ple;
	plb = ple = 1;
	for (i = 2; i < nlay; i++)
	{
		int cur = R(inlayl, inlayu);
		if (nver + cur > MAXN) 
		{
			nver += cur;
			return;
		}
		layerbe[i] = nver + 1;
		layeren[i] = nver + cur;
		// necessary edges
		for (j = nver + 1; j <= nver + cur; j++) havein[j] = 0;
		for (j = plb; j <= ple; j++)
		{
			int t = R(nver + 1, nver + cur);
			AddEdge(j, t);
			havein[t] = 1;
		}
		for (j = nver + 1; j <= nver + cur; j++)
		{
			if (havein[j] == 1) continue;
			AddEdge(R(plb, ple), j);
		}
		if (i != 2)
		{
			// leak edges
			int cae = R(minae, maxae);
			for (j = 0; j < cae; j++)
			{
				AddEdge(R(plb, ple), R(nver + 1, nver + cur));
			}
		}
		for (j = nver + 1; j <= nver + cur; j++) layer[j] = i;
		nver += cur;
		plb = layerbe[i];
		ple = layeren[i];
	}
	for (i = plb; i <= ple; i++)
	{
		AddEdge(i, 2);
	}
	numlayers = nlay;

}

void AddEdge2(int v1, int v2, int c)
{
	if (ne2 > MAXM) return;
	edg2[ne2].be = v1;
	edg2[ne2].en = v2;
	edg2[ne2].c = c;
	ne2++;
}

int GetWGood(int w)
{
	int q = RGood(NUMPOSWGT + NUMNEGWGT);
	if (q < NUMNEGWGT) return -RGood(0, w);
	else return RGood(0, w);
}

void GenAddingEdges(int numadd, int wgtmax)
{
	if (nver > MAXN) return;
	ne2 = 0;
	int i;
	for (i = 0; i < numadd; i++)
	{
		int clay = R(2, numlayers);
		int cl2;
		cl2 = R(max(1, clay - 2), clay - 1);
		int v1, v2;
		v1 = R(layerbe[clay], layeren[clay]);
		v2 = R(layerbe[cl2], layeren[cl2]);
		AddEdge2(v1, v2, GetWGood(wgtmax));
	}
}

void GenAddingEdges2(int numadd, int wgtmax)
{
	if (nver > MAXN) return;
	ne2 = 0;
	int i;
	for (i = 0; i < numadd; i++)
	{
		int clay = R(1, numlayers - 1);
		int cl2;
		cl2 = R(min(clay + 1, numlayers), numlayers);
		int v1, v2;
		v1 = R(layerbe[clay], layeren[clay]);
		v2 = R(layerbe[cl2], layeren[cl2]);
		AddEdge2(v1, v2, -GetWGood(wgtmax));
	}
}

int nverl[MAXN + 1];
int nverr[MAXN + 1];
int isleaf[MAXN + 1];

void GenSC(int pos)
{
	if (nver > MAXN) return;
	int i;
	for (i = nverl[pos]; i <= nverr[pos]; i++) isleaf[i] = 1;
	for (i = nverl[pos] + 1; i <= nverr[pos]; i++)
	{
		int trt = R(nverl[pos], i - 1);
		AddEdge(trt, i);
		isleaf[trt] = 0;
	}
	vector<int> lvs;
	lvs.push_back(nverl[pos]);
	for (i = nverl[pos] + 1; i <= nverr[pos]; i++)
	{
		if (isleaf[i] == 0) continue;
		AddEdge(i, lvs[R(0, lvs.size() - 1)]);
		lvs.push_back(i);
	}
}

void AddSC(int minsize, int maxsize)
{
	if (nver > MAXN) return;
	int oldm1 = ne;
	int i;
	int wnv = nver;
	int pnn = 0;
	for (i = 1; i <= wnv; i++)
	{
		nverl[i] = pnn + 1;
		nverr[i] = nverl[i] + R(minsize, maxsize) - 1;
		pnn = nverr[i];
	}
	if (nverr[nver] > MAXN)
	{
		nver = nverr[nver];
		return;
	}
	for (i = 1; i <= wnv; i++)
	{
		GenSC(i);
	}
	for (i = 0; i < oldm1; i++)
	{
		edg[i].be = R(nverl[edg[i].be], nverr[edg[i].be]);
		edg[i].en = R(nverl[edg[i].en], nverr[edg[i].en]);
	}
	for (i = 0; i < ne2; i++)
	{
		edg2[i].be = R(nverl[edg2[i].be], nverr[edg2[i].be]);
		edg2[i].en = R(nverl[edg2[i].en], nverr[edg2[i].en]);
	}
	nver = nverr[nver];
}

int p[MAXN + 1];

void ShuffleVert()
{
	if (nver > MAXN) return;
	int i;
	for (i = 1; i <= nver; i++)
	{
		p[i] = i;
		int j = R(1, i);
		int t = p[i];
		p[i] = p[j];
		p[j] = t;
	}
	for (i = 0; i < ne; i++)
	{
		edg[i].be = p[edg[i].be];
		edg[i].en = p[edg[i].en];
	}
	for (i = 0; i < ne2; i++)
	{
		edg2[i].be = p[edg2[i].be];
		edg2[i].en = p[edg2[i].en];
	}
}

void WriteGraph()
{
	if ((ne + ne2 > MAXM) || (nver > MAXN)) return;
	OpenNextFile();
	cout << nver << "\n";
	cout << ne << "\n";
	int i;
	for (i = 0; i < ne; i++)
	{
		cout << edg[i].be << " " << edg[i].en << "\n";
	}
	cout << ne2 << "\n";
	for (i = 0; i < ne2; i++)
	{
		cout << edg2[i].be << " " << edg2[i].en << " " << edg2[i].c << "\n";
	}
	CloseFile();
}

int main()
{
	initrand(657862);
	curtest = 0;
    GenSampleTests();
	
	int nlays[] = {3, 5, 10, 100, 1000, 10000}; 
	int grsz[] = {10, 100, 1000, 10000, 100000};
	int numsz[] = {3, 3,  5,  5, 6};
	int wgts[] = {10, 1000, 1000000, 1000000000};
	int i, j, p;
	// tests without strong components
	for (i = 0; i < 5; i++)
	{
		for (j = 0; j < numsz[i]; j++)
		{
			int k = RGood(6);
			int lszmin = grsz[i] / nlays[k];
			if (lszmin == 0) continue;
			int lszmax = lszmin + 1;
			Init();
			GenLayered(nlays[k], lszmin, lszmax, 0, rndvalue(lszmax));
			p = RGood(4);
			GenAddingEdges(5 * rndvalue(nver), wgts[p]);
			//GenAddingEdges2(5 * rndvalue(nver), wgts[p]);
			//AddSC(1, 10);
			ShuffleVert();
			WriteGraph();
		}
	} 
	return 0;
}
